import ddf.minim.*;
import ddf.minim.ugens.*;
import processing.video.*;


Minim       minim;
AudioOutput out;
Oscil       wave;


color black = color(0);
color white = color(255);
int rows = 64;
int cols = 64;
int numPixels;
Capture video;
int sz = 16;

float[][] heightMap = new float[rows][cols];
PVector ballPos = new PVector(round(random(rows)),round(random(cols)),0);
PVector cameraPos = new PVector(1000,1000,1000);
int numBalls = 10;
Ball[] balls = new Ball[numBalls];
Ball leader;

void setup() {
  size(1024, 576, P3D); 
  video = new Capture(this, rows , cols);
  video.start();  
  numPixels = video.width * video.height;
  //noCursor();
  
  smooth();
  
  createSound();
  
  for( int i = 0 ; i < numBalls ; i++){
    Ball tempBall;
    color c;
    if(i == 0){
      c = color(255,0,0);
      tempBall = new Ball(round(random(rows)), round(random(cols)), sz, sz*2, c, true);
      leader = tempBall;
    }else{
      c = color(0,0,random(128)+127);
      tempBall = new Ball(round(random(rows)), round(random(cols)), sz, sz*2, c, false);
    }
    
    balls[i] = tempBall;
  }
  
  frame.setTitle("3D2D3DBall");
}

void draw() {
  background(0);
  //drawAxis();
  noStroke();
  
  if (video.available()) {
    
    video.read();
    video.loadPixels();
    
    int i = 0;
    fill(255);
    
    directionalLight(126, 126, 126, 0, 0, -1);
    ambientLight(102, 102, 102);
    
    cameraPos.x = mouseX*2 - 512;// 512 * sin(frameCount*0.01) + 512;
    cameraPos.y = mouseY*2 + 256  ;// 256 * cos(frameCount*0.006) + 960;
    cameraPos.z = 1200;
    
    camera(cameraPos.x, cameraPos.y, cameraPos.z, 
           rows/2*sz , cols/2*sz, 256,
           0.0, 1.0, 0.0);
    
     for(int y = 0; y < cols; y++){
       for(int x = 0; x < rows; x++){
         int loc = (video.width - x - 1) + y*video.width; // Reversing x to mirror the image
         
         
         
         float boxHei = 255.0 - brightness(video.pixels[loc]);     
         heightMap[x][y] = boxHei;
         pushMatrix();
         fill(boxHei);
           translate(x*sz, y*sz,boxHei);
           box(sz,sz,boxHei*2 );
         popMatrix();
         i++;
       } 
    }
    for(i = 0 ; i < numBalls ; i++){
      
      Ball tempBall = balls[i];
      tempBall.update();
      tempBall.display();
    }
    //drawBall();
  }
}



void createSound(){
 minim = new Minim(this);
  
  // use the getLineOut method of the Minim object to get an AudioOutput object
  out = minim.getLineOut();
  
  // create a sine wave Oscil, set to 440 Hz, at 0.5 amplitude
  wave = new Oscil( 440, 0.5f, Waves.SINE );
  // patch the Oscil to the output
  wave.patch( out ); 
}

void setSoundProperties(float amp, float freq){
   //float amp = map( mouseY, 0, height, 1, 0 );
   wave.setAmplitude( amp );
  
   //float freq = map( mouseX, 0, width, 110, 880 );
   wave.setFrequency( freq );  
}


void reset(){
    for(int i = 0 ; i < numBalls ; i++){
      
      Ball tempBall = balls[i];
      tempBall.row  = round(random(rows-1));
      tempBall.col  = round(random(cols-1));
    }
}

void keyPressed(){
 if(key == ' '){
    reset();
 } 
}
